//******************************************************************************
//  Ridici program pro fm prijimac
//
//  Robert Kucera
//  OK2UWQ
//   2013
//  Built with IAR Embedded Workbench Version: 6.50
//
//******************************************************************************

#include "msp430g2553.h"

#define PowerON  0x08
#define LedStereo 0x80
#define LedPwr  0x40

#define SrData  0x20
#define SrClk   0x10
#define SrStb   0x08
#define PotCS   0x20

#define DefaultTimeout   0x08                   //0.5sec per tick
#define ExtraLongTime  0x06
#define LongTime  0x04
#define ShortTime  0x01
#define TimeBlik 0x03
#define DelayBlik 0x04
#define MinFreqCCIR  88000
#define MaxFreqCCIR  108000
#define PresetVolume 5
#define MaxVolume 40
#define MaxMenu 2

static unsigned char I2CTxCnt;
static unsigned char I2CRxCnt;
static unsigned char I2CTxPos;
static unsigned char TEAbufTx[5];
static unsigned char *PTEAbufRx;
volatile unsigned char TEAbufRx[5]; 
static unsigned char Display[4];
static unsigned char MenuId[MaxMenu+1];
static unsigned long FreqMem[10];
static unsigned char PotTxData[4];
static unsigned char PotRxData[5];
static unsigned char posSPI;
static unsigned char SPIdataSent;

static unsigned char I2CRx;
static signed char Count;
static signed char StepFreq;
unsigned char HighInject;
unsigned long Freq;
unsigned char stav;
unsigned char blokstav;
unsigned char laststav;
unsigned char updateFreq;
unsigned char updateCount;
unsigned char timeout;
unsigned char startpower;
unsigned char PowerIsON;
unsigned char TestFrequency;
unsigned char TestCnt;
unsigned char BlikPwrCnt;
unsigned char resetWDT;
unsigned char StereoDetect;
unsigned char DisplayOff;

static unsigned char FrequencySelected;
static unsigned char MenuSelected;
static unsigned char MemorySaveSelected;
static unsigned char BtnMenu;
static unsigned char BtnMenuTime;
static unsigned char Value;
static unsigned char ShowMem;

static signed char Volume;
static signed char Menu;
static signed char Memory;


// bit possition 7 to 0 - 7 segment - G F A B E D C H(dp)
//     A
//   F  B
//    G
//   E  C
//    D  H
//
static const unsigned char TabChar[]=
{0x81,0xed,0x43,0x49,0x2d,0x19,0x11,0xcd,0x01,0x09
};
static const unsigned char MsgOn[] = {0x81,0x75};  //On
static const unsigned char MsgOff[] = {0x81,0x17,0x17}; //OFF
static const unsigned char MsgVol[] = {0xb3,0x7f}; //L-
static const unsigned char MsgEnd[] = {0xff,0x13,0x75,0x61}; //End
static const unsigned char MsgMem[] = {0x75,0x77,0x7f}; //nr-

//static const unsigned char TabLogVolume[] = {0,4,9,13,17,22,26,30,34,37,41,45,48,52,55,59,62,66,69,72,75,78,81,84,87,91,93,95,98,101,104,106,109,111,114,116,119,121,124,126,128};
//static const unsigned char TabLogVolume[] = {0,2,5,8,10,13,15,17,20,22,24,26,28,30,32,34,36,38,40,42,44,46,47,49,51,52,54,56,57,59,61,62,64,65,67,68,69,71,72,74,75};
static const unsigned char TabLogVolume[] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,16,18,20,22,24,27,30,33,36,39,43,47,51,55,61,67,73,80,87,94,101,108,115,122,128,128};
void TunerInit(void);
void TunerStandby(unsigned char stby);
void TunerPreset(unsigned char mute, unsigned char setmono, unsigned long setfreq);
unsigned char ReadFrequency(void);
void ShowFreq(void);
void ShowVolume(void);
void ShowMenu(void);
void ShowMemory(void);
void InitDisplay(void);
void RefreshDisplay(void);
void PowerON_OFF(unsigned char ON);
void ReadFlash(void);
void WriteFlash(void);

void SetVolume(unsigned char level);
void PrepareVolumeData(unsigned char volume);
void WritePotNoIRQ(void);
void Delay500ms(void);

/******************************************************************************
**
**	Procedura main
**
******************************************************************************/ 
int main(void)
{
  BCSCTL1 = CALBC1_8MHZ;                        // Nastavi pracovni kmitocet DCO na 8MHz (MCLK)
  DCOCTL = CALDCO_8MHZ;
  FCTL2 = FWKEY + FSSEL0 + FN3;                 // MCLK/9 pro Flash Timing Generator, musi byt 500kHz<f<1MHz
  
  Freq = 88500;                                 //vychozi kmitocet 88.5MHz
  StepFreq = 100;                               //vychozi krok 100kHz
  HighInject = 1;                               //shorni postranni pasmo v TEA5767
  startpower = 0;                               //navesti pro zapnuti/vypnuti napajeni    
  Volume = PresetVolume;
  
  MemorySaveSelected = 0;                       //nulovani navesti...
  MenuSelected = 0;
  FrequencySelected = 0;

  TestCnt = DelayBlik;                          //prodleva blikani cervene LED

  BCSCTL3 |= LFXT1S_2;                          // LFXT1 = VLO
  WDTCTL = WDT_ARST_1000;                       // ~2.7s  pri ACLK = VLO

  P1SEL = BIT1 + BIT2 + BIT4 + BIT6 + BIT7;     // SPI piny na USCI_A0, I2C piny na USCI_B0
  P1SEL2 = BIT1 + BIT2 + BIT4 + BIT6 + BIT7;    // SPI piny na USCI_A0, I2C piny na USCI_B0
  
  P1DIR |= PowerON + PotCS;                     // P1.3, P1.5 vystup
  P1OUT |= PotCS;                               //deaktivace pinu CS MCP4241
  P1OUT &= ~PowerON;                            // nulovani vystupu - power off .
  
  P2SEL = 0x00;
  P2DIR = 0xf8;                                 // P2.0-2.2 vstupy
  P2REN = 0x07;                                 // pullup rezistory zapnuty
  P2OUT = 0xc7;                             

  TA0CTL = TASSEL_1 + MC_1;                     // ACLK, up mode citace 0
  TA1CTL = TASSEL_1 + MC_1;                     // ACLK, up mode citace 1

  TA1CCR0 = 6000;                               // PWM Perioda 0.5sec
  
  TA0CCR0 = 2000;                               // PWM Perioda 0.15sec
  while(!(CCTL0&CCIFG));                        // ceka na dokonceni periody citace    
  CCTL0 &= ~CCIFG;
  WDTCTL = WDT_ARST_1000;                       // nulovani watchdog citace

  P1OUT |= PowerON;                             //PWR on - zapnuti napajeni
  InitDisplay();                                //zhasnuti displeje a zelene LED
  Display[1] = MsgOff[0];                       //nacteni zpravy OFF pro displej
  Display[2] = MsgOff[1];
  Display[3] = MsgOff[2];
  RefreshDisplay();                             //zobrazeni na displeji
  WDTCTL = WDT_ARST_1000;                       // nulovani watchdog citace
  TA0CCR0 = 6000;                               // PWM Perioda 0.5sec (kmitocet VLO 12000Hz / 6000)

  while(!(CCTL0&CCIFG));                        // ceka na dokonceni periody citace - zpozdeni 0.5sec
  CCTL0 &= ~CCIFG;
  WDTCTL = WDT_ARST_1000;                       // nulovani watchdog citace
  
  InitDisplay();                                //zhasnuti displeje - napisu OFF
  WDTCTL = WDT_ARST_1000;                    

  ReadFlash();                                  //cteni Flash pameti s udaji o pametech a nastavenich
  ShowMem = MenuId[1];                          //nacteni parametru povoleni prepinani a zobrazeni pameti

//Enable SPI module
  
  UCA0CTL0 |= UCCKPL + UCMSB + UCMST + UCSYNC;  // 3-pin, 8-bit SPI master
  UCA0CTL1 |= UCSSEL_2;                         // SMCLK
  UCA0BR0 |= 0x02;                              // /2
  UCA0BR1 = 0;                                  //
  UCA0MCTL = 0;                                 // bez monulace
  UCA0CTL1 &= ~UCSWRST;                         // Initializace USCI modulu nulovanim Reset
//  IE2 |= UCA0RXIE;                            // Enable USCI0 RX interrupt   
  
  posSPI = 4;                                   //nastavei ukazatele na posledni byte
  SPIdataSent = 0;
  
  SetVolume(0);                                 //nastaveni minimalni hlasitosti
  
//Enable I2C module

  UCB0CTL1 |= UCSWRST;                          // SW reset modulu
  UCB0CTL0 = UCMST + UCMODE_3 + UCSYNC;         // I2C Master, synchronous mode
  UCB0CTL1 = UCSSEL_2 + UCSWRST;                // Use SMCLK, zachovani SW reset
  UCB0BR0 = 20;                                 // fSCL = SMCLK/20 = ~400kHz
  UCB0BR1 = 0;
  UCB0I2CSA = 0x60;                             // slave address - TEA5767
  UCB0CTL1 &= ~UCSWRST;                         // nulovani SW resetu - uvolneni modulu
  IE2 |= UCB0TXIE | UCB0RXIE;                   // povoli Rx+TX preruseni
  I2CRx = 0;
  
  __bis_SR_register(GIE);                       // povoli vsechna povolena preruseni -general interrupt enable

  TA0CCR0 = 24;                                 // PWM Perioda 2ms (500Hz)
  TA0CCTL0 = CCIE;                              // povoli preruseni citace 0
  TA1CCTL0 = CCIE;                              // povoli preruseni citace 1

  TunerStandby(1);                              //prepnuti TEA5767 do standby modu
  while(I2CTxCnt);                              //ceka na dokonceni vysilani prikazu po I2C
  WDTCTL = WDT_ARST_1000;                       // nulovani watchdog citace

  P1OUT &= ~PowerON;                            //PWR off - vypnuti audio zesilovace a TEA modulu
  
  while(1){                                     //hlavni smycka programu - provede se pouze pokud neni CPU v LPMx modu
    if(updateCount)                             // zmena inkrementalniho snimace? otestuj moznosti
    {
      if(PowerIsON)                             //pokud je FM radio zapnuto testuj jednotlive mody
      {
        if(FrequencySelected&(MenuSelected==0)&(MemorySaveSelected==0)) //pokud je mod zobrazeni pameti/kmitoctu
        {
          if(ShowMem)                           // pokud zobraz pamet
          {
            Memory += Count;                    // pricteni poctu kroku snimace
            if(Memory > 9)                      // pokud pamet>9 pak prechod na pamet 0
            {
              Memory = 0;
            }else if(Memory < 0)Memory = 9;     // pokud mensi jak 0, prechod na pamet 9
            
            TestFrequency = 0;                  // vypnuti cteni stavu modulu TEA5767 pri preladeni
            P2OUT |= LedStereo;                 // vypnuti signalizace Stereo
            StereoDetect = 0;                   // nulovani priznaku stereo
            Freq = FreqMem[Memory];             // nacteni pozadovaneho kmitoctu ze zvolene pameti
            TunerPreset(0,MenuId[2],Freq);      // preladeni modulu TEA5767 na pozadovany kmitocet
            ShowMemory();                       // zobrazeni cisla pameti
          }else
          {                                     // pokud neni zobrazeni pameti - pak zobrazeni kmitoctu
            Freq += Count*StepFreq;             // pricteni/odecteni kitoctu o poctu kroku snimace nasobenych kmitoct. krokem
            if(Freq > MaxFreqCCIR)              // pokud je nastaveny kmitocet vetsi jak max. CCIR (108MHz) 
            {
              Freq = MinFreqCCIR;               // nastav minimalni kmitocet CCIR pasma (88MHz)
            }else if(Freq < MinFreqCCIR)        // pokud mensi kmitocet jak min CCIR
            {
              Freq = MaxFreqCCIR;               // nastav max CCIR kmitocet
            }
            TestFrequency = 0;                  // vypnuti cteni stavu modulu TEA5767 pri preladeni
            P2OUT |= LedStereo;                 // vypnuti signalizace Stereo
            StereoDetect = 0;                   // nulovani priznaku stereo
            TunerPreset(0,MenuId[2],Freq);      // preladeni modulu TEA5767 na pozadovany kmitocet
            ShowFreq();                         // zobrazi kmitocet
          }
        }else if(MenuSelected)                  // pokud je vybrano menu
        {
          if(FrequencySelected==0)              // pokud nedoslo ke kratkemu stisku tlacitka inkrement. snimace (jako pro mod zobr. kmitoctu/pameti)
          {
            Menu += Count;                      // posun na pozadovanou polozku menu
            if(Menu > MaxMenu)                  // test na max pocet polozek menu
            {
              Menu = 0;
            }else if(Menu < 0)                  // test na min pocet polozek menu
            {
              Menu = MaxMenu;
            }
            ShowMenu();                         // zobrazi menu
          }else                                 // pokud kratky stisk tlacitka pak zmena hodnoty polozky menu
          {
            if(Value==0)
            {
                Value++;
            }else Value = 0;                    // umoznuje nabyvat pouze hodnoty 0/1 (off/on)stridave po stiku tlacitka
            MenuId[Menu] = Value;               // ulozeni hodnoty polozky menu
            FrequencySelected = 0;              // ukonceni zadavani hodnoty
            if(Menu == 0)                       // pokud je polozka 0 menu, pak se jedna o volbu opusteni menu
            {
              MenuSelected = 0;                 // ukonci menu
              WriteFlash();                     // ulozi hodnoty do Flash - timto se ulozi nejen nastaveni, ale i zvolena pamet a obsahy pameti
              InitDisplay();                    // vypne displej
            }else ShowMenu();                   // pokud neni End pak zobrazi menu se zmenenou hodnotou
          }
        }else if(MemorySaveSelected)            // pokud je zvoleno ukladani do pameti
        {
          if(FrequencySelected)                 // pokud stisknuto tlacitko pak prechod na ulozeni pameti
          {
            FreqMem[Memory] = Freq;             // ulozi kmitocet do vybrane pameti
            WriteFlash();
            FrequencySelected = 0;
            MemorySaveSelected = 0;
            InitDisplay();                      // vypne displej
          }else{                                // pokud nebylo stisknuto tlacitko pak prepinani pameti
            Memory += Count;            
            if(Memory>9)
            {
              Memory = 0;
            }else if(Memory<0)Memory = 9;
            ShowMemory();                       // zobrazeni cisla zvolene pameti
          }
        }else                                   // pokud nebyl zadny stisk tlacitka snimace pak volba hlasitosti
        {
          Volume += Count;                      
          if(Volume > MaxVolume)                // test na max. moznou hlasitost
          {
            Volume = MaxVolume;
          }else if(Volume < 0)                  // test na min. hlasitost
          {
            Volume = 0;
          }
          SetVolume(Volume);                    // nastaveni hlasitosti
          ShowVolume();                         // zobrazeni hlasitosti
        }
      }
      Count = 0;                                // nulovani poctu 
      updateCount = 0;
    }
    if(startpower)                              // pokud zmena napajeni (vyp/zap)
    {  
      startpower = 0;                           // nulovani priznaku zmeny
      PowerON_OFF(PowerIsON);                   // zavolani funkce zapnuti/vypnuti
    }
    if(TestFrequency)                           // pokud nastaveno cteni modulu TEA5767
    {
      TestFrequency = 0;                        // nulovani priznaku
      ReadFrequency();                          // cteni parametru modulu
      while(I2CRx);                             // ceka na dokonceni cteni sbernice I2C
      if(DisplayOff==0)P2OUT |= LedStereo;      // pokud neni vypnuty displej vypni signalizaci Stereo
      StereoDetect = 0;                         // a nuluj priznak Stereo
      if((TEAbufRx[2]&0x80)==0x80) 
      {
        if(MenuId[2]==0)                        // pokud je vynucene mono nerozsvecet stereo LED ani nenastavovat priznak
        {
          if(DisplayOff==0)P2OUT &= ~LedStereo; // pokud je Stereo detekovano a displej zapnuty, zapni signalizacni zelenou LED
          StereoDetect = 1;                     // jinak pouze nastav priznak Stereo (pouzit pri zapnuti displeje k zapnuti LED)
        }
      }
    }
    if(resetWDT)                                // pokud nastaven pozdavek na reset watchdog citace
    {
      resetWDT = 0;
      WDTCTL = WDT_ARST_1000;                   // nuluje citac watchdog
    }
    
    __bis_SR_register(LPM3_bits + GIE);         // zapne low power mode LPM3 a povoli preruseni... dalsi vstup do hlavni smycky v okamziku ukonceni LPMx modu
  }
}

/******************************************************************************
**
**	Procedura preruseni od SPI modulu
**
******************************************************************************/ 
/*                                               //obsluha preruseni od SPI momentalne nevyuzita - zmena hlasitosti se provadi mimo preruseni
// USCI_A0  SPI to pot MCP4241
#pragma vector=USCIAB0RX_VECTOR
__interrupt void USCIA0RX_ISR(void)
{
  volatile unsigned int i;

  while (!(IFG2 & UCA0TXIFG));                  // USCI_A0 TX buffer ready?

  while(posSPI<4)
  {
    UCA0TXBUF = PotTxData[posSPI++];            // posli dalsi hodnotu
    if(posSPI==4)
    {
      SPIdataSent = 1;                          // nastav priznak odeslani vsech dat
    }
  }
  PotRxData[posSPI] = UCA0RXBUF;                // prijate data
  __delay_cycles(50);                           // pridej prodlevu pri odvysilani pro umozneni zpracovani dat podrizene strane
}
*/

/******************************************************************************
**
**	Procedura preruseni od I2C modulu
**
******************************************************************************/ 
// USCI_B0 Data ISR                             // zpracovani preruseni od modulu I2C
#pragma vector = USCIAB0TX_VECTOR
__interrupt void USCIAB0TX_ISR(void)
{
  if(I2CRx)                                     // pokud je prijem dat z modulu TEA5767
  {
    I2CRxCnt--;                                 // pocet ocekavanych dat -1
    if(I2CRxCnt)                                // pokud pocet ocekavanych dat >0
    {
      *PTEAbufRx++= UCB0RXBUF;                  // ulozi prijate data do bufferu podle ukazatele
      if (I2CRxCnt == 1)                        // jedna se o posledni byte, ktery jeste ma byt prijat?
      {  
        UCB0CTL1 |= UCTXSTP;                    // Generuj I2C stop
      }
    }else{                                      // jinak se jedna o posledni byte
      *PTEAbufRx++= UCB0RXBUF;                  // presun posledniho byte do bufferu
      I2CRx = 0;                                // nulovani navesti prijmu dat po i2c

    }
  }
  if(I2CTxCnt)                                  // pokud je vysilani dat po I2C
  {
    I2CTxCnt--;                                 // pocet dat k vyslani -1
    if(I2CTxCnt==0)UCB0CTL1 |= UCTXSTP;         // I2C stop pri poslednim byte
    UCB0TXBUF = TEAbufTx[I2CTxPos++];           // vysilani data bytu
  }
} 

/******************************************************************************
**
**	Procedura preruseni od citace 0
**
******************************************************************************/ 
// Timer A0 interrupt service routine           // zpracovani preruseni od citace 0 - pracovni citac pro testovani smeru posuvu inkrementalniho snimace
#pragma vector=TIMER0_A0_VECTOR
__interrupt void Timer_A (void)
{
  stav = (P2IN&0x06);                           // vymaskovani vstupnich pinu mimo piny, kde je pripojeny snimac

  switch(stav){
    case 0:{                                    // pokud 0b00
         blokstav=0;                            // nuluje priznak blokovani stavu - blok. opakovaneho zpracovani stavu bez zmeny
      }break;
    case 2:{
      }break;
    case 4:{
      }break;
    case 6:{
         if(blokstav==0){                       // pokud 0b11 a neni blokovano vyhodnoceni
           if(laststav==0x02){                  // pokud posledni stav 0b01
             Count--;                           // hodnota -1
             blokstav=1;                        // blokuj dalsi vyhodnoceni do dalsi zmeny
             updateCount = 1;                   // nastav priznak vyhodnoceni v hlavni smycce
           }else if(laststav==0x04){
             blokstav=1;                        // blokuj dalsi vyhodnoceni do dalsi zmeny
             Count++;                           // hodnota +1
             updateCount = 1;                   // nastav priznak vyhodnoceni v hlavni smycce
           }
         }
      }break;
  }
  laststav=stav;                                // presun aktualniho stavu pro nasledujici vyhodnoceni behem dalsiho preruseni
  if((P2IN&0x01)==0)                            // test stisku tlacitka na snimaci
  { 
    if(BtnMenu==0)                              // pokud je prvni detekce stisku v tomto pruchodu prerusenim
    {
      BtnMenu = 1;                              // nastav priznak stisku
      BtnMenuTime=0;                            // nuluj pocitadlo casu stisknuti
    }
  }else if(BtnMenu){                            // pokud jiz bulo stisknuto tlacitko a nyni je poprve detekce uvolneni tlacitka
    if((BtnMenuTime>LongTime)&(BtnMenuTime<=ExtraLongTime))     // testuj dobu stisknuti
    {                   
      MenuSelected = 1;                         // pokud doba >longTime a zaroven <Extralongtime pak se jedna o pozadavek vstupu do nabidky Menu
      MemorySaveSelected = 0;                   // nuluje se pozadavek na ukladani do pameti
    }else if(BtnMenuTime>ShortTime)             // pokud je doba >ShortTime a <LongTime
    {
      MemorySaveSelected = 1;                   // pozadavek na ukladani do pameti
    }else{
      if(FrequencySelected&MenuId[1])           // pokud je doba <= Shorttime pak pokud je povoleno zobrazeni pameti
      {
        if(ShowMem==0)                          // pokud byl zobrazen kmitocet 
        {
          ShowMem++;                            // zobraz pamet
        }else ShowMem=0;                        // jinak zobraz kmitocet
      }
      FrequencySelected = 1;                    // nastav priznak zobrazeni kmitoctu/pameti
    }
    updateCount = 1;                            // nastav priznak vyhodnoceni v hlavni smycce
    Count = 0;                                  // pak nuluj pozici snimace
    BtnMenu = 0;                                // nuluj priznak stisknuteho tlacika
    BtnMenuTime=0;                              // nuluj cas stisknuti
  }
  if(BlikPwrCnt)                                // citac doby svitu cervene LED, pokud je >0
  {
    BlikPwrCnt--;                               // citac doby svitu -1
    if(BlikPwrCnt==0)                           // pokud je 0
    {
      P2OUT |= LedPwr;                          // zhasnuti cervene LED
    }
  }
  if(updateCount)  __bic_SR_register_on_exit(LPM3_bits);      // pokud pozadavek na zpracovani v hlavni smycce programu.. opusteni LPMx
}

/******************************************************************************
**
**	Procedura preruseni od citace 1
**
******************************************************************************/ 
// Timer A1 interrupt service routine           // zpracovani preruseni od citace 1 - doba 0.5sec
#pragma vector=TIMER1_A0_VECTOR
__interrupt void Timer1_A (void)
{
  if(timeout)                                   // pocitadlo timeout pro zhasnuti displeje - pokud >0 odpocitavej
  {
    timeout--;                                  // -1
    if(timeout==0)                              // pokud je 0
    {
      InitDisplay();                            // zhasni displej
      FrequencySelected = 0;                    // vynuluj priznak zobrazeni kmitoctu
      MemorySaveSelected = 0;                   // vynuluj priznak volby ulozeni do pameti
      MenuSelected = 0;                         // vynuluj priznak volby menu
      ShowMem = MenuId[1];                      // nastav vychozi zobrazeni 0 - kmitocet, 1 - cislo pameti
      Count = 0;                                // nuluj pozici snimace
    }
  }
  if(BtnMenu)                                   // pokud stisk tlacitka
  {
    BtnMenuTime++;                              // pocitadlo casu +1
    if(PowerIsON&(BtnMenuTime>LongTime)) P2OUT &= ~LedPwr;      // pokud FM radio zapnuto a cas >LongTime, pak rozsvit cervenou LED
    if(BtnMenuTime>ExtraLongTime)               // pokud cas > ExtraLiongTime pak pozadavek na zapnuti/vypnuti FM radia
    {
      BtnMenu = 0;
      if(PowerIsON)                             // pokud je zapnuto
      {
        PowerIsON=0;                            // nulovani priznaku
        PowerON_OFF(PowerIsON);                 // vypnuti radia
      }else                                     // jinak zapnuti
      {
        PowerIsON = 1;                          // nastaveni priznaku
        startpower=1;                           // nastaveni pozadavku na zapnuti (nelze zapinat v rutine preruseni kvuli dlouhe dobe inicializace pri zapnuti)
        __bic_SR_register_on_exit(LPM3_bits);   // opusteni LPMx a nasledne po ukonceni obsluhy preruseni vstup do hlavni smycky
      }
    }
  }
  TestCnt--;                                    // pocitadlo prodlevy blikani cervene led pokud je zapnute radio -1
  if(TestCnt==0)
  {
    TestCnt = DelayBlik;                        // pokud je 0, pak nacteni znova vychoziho casu
    resetWDT = 1;                               // pozadavek na nulovani watchdogu v hlavni smycce
    if(PowerIsON)                               // pokud je zapnuto FM radio
    {
      TestFrequency = 1;                        // nastaveni pozadavku na ctenis stavu modulu TEA5767
      BlikPwrCnt= TimeBlik;                     // nastaveni doby svitu LED
      P2OUT &= ~LedPwr;                         // rozsviceni cervene LED
    }
    __bic_SR_register_on_exit(LPM3_bits);       // opusteni LPMx
  }
 
}

/******************************************************************************
**
**	Procedura TunerInit
**
******************************************************************************/ 
void TunerInit(void)                            // zajisti inicializaci modulu TEA5767 - uvodni nastaveni viz datasheet tohoto IC
{
  TEAbufTx[0] = 0xad;
  TEAbufTx[1] = 0xef;
  TEAbufTx[2] = 0x41;
  TEAbufTx[3] = 0x19;
  TEAbufTx[4] = 0x40;
  I2CTxPos = 0;
  I2CTxCnt = 5;                                 //Pocet bytu k odeslani
  UCB0CTL1 |= UCTR + UCTXSTT;                   // I2C TX, start condition
}

/******************************************************************************
**
**	Procedure TunerStandby
**
******************************************************************************/ 
void TunerStandby(unsigned char stby)           // uvedeni modulu do standby nebo opusteni standby
{
  TEAbufTx[3] &= ~0x40;
  if(stby)TEAbufTx[3] |= 0x40;          
  I2CTxPos = 0;
  I2CTxCnt = 5;                                 //Pocet bytu k odeslani
  UCB0CTL1 |= UCTR + UCTXSTT;                   // I2C TX, start condition
}

/******************************************************************************
**
**	Procedure TunerPreset
**
******************************************************************************/ 
void TunerPreset(unsigned char mute, unsigned char setmono, unsigned long setfreq)      // nastaveni tuneru na pozadovany kmitocet, vcetne doplnkovych funkci (mute, mono)
{
  unsigned long tmp;
  WDTCTL = WDT_ARST_1000;                       // nulovani citace watchdogu
  tmp = setfreq;                                // nasleduje prevod kmitoctu do potrebne formy
  if(HighInject)                                // pokud spodni postranni pasmo
  {
    tmp +=225;                                  // pricte IF
  }else{
    tmp -=225;                                  // jinak odecte IF (225kHz)
  }
  tmp *=125;                                    // nutny vypocet hodnoty pro PLL pri referencnim krystalu 32768Hz - celociselny - freq*4 /32.768
  tmp /= 1024;                                  // *125 / 1024 odpovida *4 /32.768
  TEAbufTx[0] = (unsigned char)(tmp >> 8);      // horni byte predvolby PLL
  TEAbufTx[1] = (unsigned char)(tmp % 256);     // spodni byte predvolby PLL
  if(mute) TEAbufTx[0] |= 0x80;                 // pokud je mute pak nastav bit
  TEAbufTx[2] = 0x41;                           // predvolba nastaveni
  if(setmono) TEAbufTx[2] |= 0x08;              // pokud ma byt mono nastav bit
  if(HighInject)TEAbufTx[2] |= 0x10;            // pokud je spodni postranni pasmo nastav bit
  TEAbufTx[3] = 0x19;
//  TEAbufTx[4] = 0x40;                         // deemphase 75us
  TEAbufTx[4] = 0x00;                           // deemphase 50us
  I2CTxPos = 0;
  I2CTxCnt = 5;                                 // pocet bytu k vyslani na I2C
  UCB0CTL1 &= ~UCSWRST;                         // uvolneni modulu I2C z reset stavu, poklud se nachazi v SW reset
  UCB0CTL1 |= UCTR + UCTXSTT;                   // I2C TX, start condition
  while(I2CTxCnt);                              // ceka na odvysilani dat
  WDTCTL = WDT_ARST_1000;                       // nulovani citace watchdogu
}

/******************************************************************************
**
**	Funkce ReadFrequency
**
******************************************************************************/ 
unsigned char ReadFrequency(void)               // cteni stavu a kmitoctu modulu TEA5767
{
  WDTCTL = WDT_ARST_1000;                       // nulovani citace watchdogu
  PTEAbufRx = (unsigned char *)TEAbufRx;        // inicializace ukazatele na buffer pro ukladani dat
  I2CRxCnt = 5;                                 // pocet ocekavanych bytu k prijmu
  I2CRx = 1;                                    // navesti cteni z I2C
  while (UCB0CTL1 & UCTXSTP);                   // cekej dokud nebyl vyslan stop
  UCB0CTL1 &= ~UCTR;                            // I2C RX 
  UCB0CTL1 |= UCTXSTT;                          // I2C start
  return 0;
}

/******************************************************************************
**
**	Procedura ShowFreq
**
******************************************************************************/ 
void ShowFreq(void)                             // zobrazi kmitocet na displeji
{
  unsigned long tmp;
  unsigned char d1,d2,d3,d4;
  tmp = Freq/100;                               // kmitocet v kHz - nebudou se zobrazovat posledni 2 mista
  d1 = (unsigned char)(tmp%10);                 // stovky kHz do d1
  tmp /= 10;                                    // deleni kmitoctu 10 - nejnizsi misto MHz
  d2 = (unsigned char)(tmp%10);                 // jednotky MHz do d2
  tmp /= 10;                                    // deleni 10 - nejnizsi misto 10MHz
  d3 = (unsigned char)(tmp%10);                 // desitky MHz do d3
  tmp /= 10;                                    // deleni 10 - 100MHz
  d4 = (unsigned char)(tmp%10);                 // stovky MHz do d4
  if(d4)                                        // kdyz d4>0
  {
    Display[0] = TabChar[d4];                   // nacti z tabulky hodnoty pro LED k rozsviceni pozadovaneho znaku
  }else Display[0] = 0xff;                      // jinak nech vsechny LED na tomto miste zhasnute
  Display[1] = TabChar[d3];                     // nacti hodnoty pro misto desitek MHz
  Display[2] = TabChar[d2]&0xfe;                // nacti pro jednotky MHz + des. tecka
  Display[3] = TabChar[d1];                     // nacti pro stovky kHz
  RefreshDisplay();                             // posle data na displej
  timeout = DefaultTimeout;                     // zacni pocitat dobu zobrazeni na displeji - kvuli uspore odberu
}

/******************************************************************************
**
**	Procedura ShowVolume
**
******************************************************************************/ 
void ShowVolume(void)                           // zobrazi nastavenou hlasitost (v krocich 0-40 pokud neni nastaveno jinak)
{
  unsigned char d1,d2, tmp;
  tmp = Volume;                                 // hodnota volume k zpracovani
  d1 = (unsigned char)(tmp%10);                 // jednotky do d1
  tmp /= 10;                                    // hodnota deleno 10
  d2 = (unsigned char)(tmp%10);                 // desitky do d2
  if(d2)                                        // pokud d2>0
  {
    Display[2] = TabChar[d2];                   // zobrazi hodnotu desitek
  }else Display[2] = 0xff;                      // jinak ponecha misto nezobrazeno
  Display[3] = TabChar[d1];                     // zobrazi jednotky
  Display[0] = MsgVol[0];                       // zobrazi "L" od slova "Level" - V se neda na 7-segmenu zobrazit
  Display[1] = MsgVol[1];                       // zobrazi "-"
  RefreshDisplay();                             // posle data na displej
  timeout = DefaultTimeout;                     // zacni pocitat dobu zobrazenina displeji
}

/******************************************************************************
**
**	Procedura ShowMemory
**
******************************************************************************/ 
void ShowMemory(void)                           // zobrazi cislo zvolene pameti
{
  unsigned char d1, tmp;
  tmp = Memory;
  d1 = (unsigned char)(tmp%10);                 // pouze jednotky - cislo 0-9
  Display[3] = TabChar[d1];
  if(MemorySaveSelected)                        // pokud je zavolano v modu ukladani
  {
    Display[0] = TabChar[5];                    // zobrazi "Sr-"
  }else Display[0] = MsgMem[0];                 // jinak "nr-"
  Display[1] = MsgMem[1];
  Display[2] = MsgMem[2];
  RefreshDisplay();                             // posle data na displej
  timeout = DefaultTimeout;                     // zacni pocitat dobu zobrazenina displeji
}

/******************************************************************************
**
**	Procedura ShowMenu
**
******************************************************************************/ 
void ShowMenu(void)
{
  unsigned char d1,tmp;
  if(Menu>0)                                    // pokud polozka menu >0
  {
    tmp = Menu;
    d1 = (unsigned char)(tmp%10);               // pouze jednotky
    Display[0] = TabChar[d1];
    Display[1] = 0x7f;                          //"-"             
    if(MenuId[Menu])                            // pokud hodnota v menu >0
    {
    Display[2] = MsgOn[0];                      // zobrazi "On"
    Display[3] = MsgOn[1];
    }else{
    Display[2] = MsgOff[0];                     // jinak zobrazi "OFF"
    Display[3] = MsgOff[1];
    }
  }else{                                        // pokud polozka menu = 0 - pak zobrazi " End"
    Display[0] = MsgEnd[0];
    Display[1] = MsgEnd[1];
    Display[2] = MsgEnd[2];
    Display[3] = MsgEnd[3];
  }
  RefreshDisplay();                             // posle data na displej
  timeout = DefaultTimeout;                     // zacni pocitat dobu zobrazenina displeji
}

/******************************************************************************
**
**	Procedura InitDisplay
**
******************************************************************************/ 
void InitDisplay(void)                          // zhasne cely displej
{
  unsigned char i;
  for(i=0;i<4;i++)
  {
    Display[i]=0xff;                            // vymaze cely buffer pro zobrazeni na displeji
  }
  RefreshDisplay();                             // posle data na displej cimz vsechny segmenty zhasnou
  P2OUT |= LedStereo;                           // zhasne take zelenou LED pro signalizaci Stereo
  DisplayOff = 1;                               // nastavi priznak zhasnuteho displeje
}

/******************************************************************************
**
**	Procedura RefreshDisplay
**
******************************************************************************/ 
void RefreshDisplay(void)                       // zajisti seriovy presun dat do registu pro displej
{
  unsigned char pos,disp,j,k;
  P2OUT &= ~SrStb;                              // signal strobe na 0
  disp = 3;                                     // prvni odesilana bude posledni pozice
  for(j = 0;j<4;j++)                            // smycka probehne 4x
  {
    k = 0x80;                                   // nejvyssi bit na 1
    for(pos = 0;pos<8;pos++)                    // smycka poctu bitu
    {
      P2OUT &= ~SrClk;                          // seriovy hodinovy signal na L
      P2OUT &= ~SrData;                         // data na L
      if(Display[disp]&k)P2OUT |= SrData;       // pokud pozadovany bit je 1 pak data na 1
      P2OUT |= SrClk;                           // hodiny na 1
      k = k>>1;                                 // posun bitu o 1pozici vpravo
    }
    disp--;                                     // po 8 pruchodech snizeni ukazatele pozice o 1
  }                                             // konec 4 x 8 pruchodu
  P2OUT |= SrStb;                               // strobe signal na 1, tim se zapisou data na vystup a rozsviti segmenty displeje podle prenesenych dat
  P2OUT &= ~SrStb;                              // strobe na L
  DisplayOff = 0;                               // nulovani ukazatele vypnuteho displeje
  if(StereoDetect)P2OUT &= ~LedStereo;          // pokud je stereo, pak rozsvit take zelenou LED
}

/******************************************************************************
**
**	Procedura PowerON_OFF
**
******************************************************************************/ 
void PowerON_OFF(unsigned char ON)
{
  if(ON)
  {
    WDTCTL = WDT_ARST_1000;                     // nulovani citace watchdogu
    
    P1OUT |= PowerON;                           // pin PWR na H - zapnuti obvodu pro audio zesilovac a modul
    PowerIsON = 1;                              // nastavi priznak zapnuti
    InitDisplay();                              // zhasne displej
    Display[1] = MsgOn[0];                      // nastavi zpravu "On" pro displej
    Display[2] = MsgOn[1];
    RefreshDisplay();                           // posle data na displej
    SetVolume(0);                               // nastavi minimalni hlasitost - nez dojde k preladeni na pozad. kmitocet omezi sum
    Delay500ms();                               // ceka 1sec pro nabehnuti napeti a inicializaci modulu
    Delay500ms();
    TunerInit();                                // zasle vychozi nastaveni modulu TEA5767
    Delay500ms();                               // pocka 500ms k zaveseni PLL
    TunerPreset(1,0,Freq);                      // nastavi pozadovany kmitocet a mute (preladeni)
    WDTCTL = WDT_ARST_1000;                     // nulovani citace watchdogu
    SetVolume(Volume);                          // nastavi pozadovanou hlasitost
  
    Delay500ms();                               // 
    TunerPreset(0,MenuId[2],Freq);              // uvolni mute - kmitocet zustava zachovan
    ShowFreq();                                 // po zapnuti zobrazi kmitocet nebo pamet na displeji

  }else{
    InitDisplay();                              // zhasne displej
    Display[1] = MsgOff[0];                     // nastavi zpravu "OFF"
    Display[2] = MsgOff[1];
    Display[3] = MsgOff[2];
    RefreshDisplay();                           // posle data na displej

    Delay500ms();                               // vycka 0.5sec - po tuto dobu je zobrazena hlaska OFF na displeji
    InitDisplay();                              // zhasne displej
    SetVolume(0);                               // nastavi hlasitost na minimum
    P1OUT &= ~PowerON;                          // nastavi pin PWR do 0 - odpoji napajeni audio a prepne el. potenciometr do standby
    P2OUT |= LedPwr | LedStereo;                // zhasne LEDky
    PowerIsON = 0;                              // nastavi priznak na 0
    WDTCTL = WDT_ARST_1000;                     // nulovani citace watchdogu

  }
}

/******************************************************************************
**
**	Procedura ReadFlash
**
******************************************************************************/ 
void ReadFlash(void)
{
  char *Flash_ptr;                              // Flash ukazatel
  unsigned long tmp;
  unsigned char i,d1,d2,d3,d4;
  Flash_ptr = (char *) 0x1000;                  // Inicializuje ukazatel na Flash segment D
  for(i=1;i<(MaxMenu+1);i++)                    // nacte stav nastaveni polozek menu
  {
    MenuId[i] = *Flash_ptr++;
  }
  Memory = *Flash_ptr++;                        // nacte vychozi zvolenou pamet
  if(Memory < 0)                                // pokud je obsah flash <0 (obvykle cista pamet obsahuje -1 tj. 0xff)
  {
    MenuId[0] =0;                               // provede inicializaci vsech promennych - obvykle pri prvnim zapnuti po naprogramovani CPU
    MenuId[1] =1;
    MenuId[2] =0;
    Memory =0 ;                                 // inicializace an pamet 0
    for(i=1;i<(MaxMenu+1);i++)
    {
      FreqMem[i] = MinFreqCCIR;                 // inicializace na kmitocet vsech pameti 88MHz
    }
    Freq = 88500;                               // vychozi nastaveni kmitoctu 88.5MHz
  }else{                                        // pokud jiz probehl zapis do pameti Flash, pak nacteni pameti
    for(i=0;i<10;i++)
    {
      d1 = *Flash_ptr++;                        // ulozene data jsou po 8bitech, nutne znova poskladat na platnou 32bit hodnotu
      d2 = *Flash_ptr++;
      d3 = *Flash_ptr++;
      d4 = *Flash_ptr++;
      tmp = (d4 <<8);                           // posun o 8vlevo odpovida nasobeni *256 jen je rychlejsi
      tmp += d3;                                // pricteni dalsiho byte
      tmp = tmp<<8;                             // opet *256
      tmp += d2;
      tmp = tmp<<8;
      tmp += d1;
      FreqMem[i] = tmp;                         // 32bit hodnota = d1 + (256*d2)+(65536*d3)+(16777216*d4)
    }
  }
}

/******************************************************************************
**
**	Procedura WriteFlash
**
******************************************************************************/ 
void WriteFlash(void)
{
  char *Flash_ptr;                              // Flash ukazatel
  unsigned long tmp;
  unsigned char i,d1,d2,d3,d4;
  Flash_ptr = (char *) 0x1000;                  // Inicializace ukazatele na Flash segment D
  FCTL1 = FWKEY + ERASE;                        // nastavi Erase bit s heslem pro zapis/mazani
  FCTL3 = FWKEY;                                // uvolni blokovani zapisu heslem
  *Flash_ptr = 0;                               // prazdny zapis do Flash zajisti vymazani celeho segmentu
  FCTL1 = FWKEY + WRT;                          // nastvi WRT bit s heslem pro zapis do pameti
  for(i=1;i<(MaxMenu+1);i++)                    // ulozi parametry menu
  {
    *Flash_ptr++ = MenuId[i];
  }
  *Flash_ptr++ = Memory;                        // ulozi cislo vybrane pameti
  for(i=0;i<10;i++)                             // ulozi obsahy pameti do flash
  {
    tmp = FreqMem[i];                           // provede rozklad 32bit cisla na 4x8bitu
    d1 = (unsigned char)(tmp%256);              // nejnizsich 8bitu do d1
    tmp = tmp >> 8;                             // /256
    d2 = (unsigned char)(tmp%256);              // nejnizsich 8bitu do d2
    tmp = tmp >> 8;                             // /256
    d3 = (unsigned char)(tmp%256);              // nejnizsich 8bitu do d3
    tmp = tmp >> 8;                             // /256
    d4 = (unsigned char)(tmp%256);              // nejnizsich 8bitu do d4
    *Flash_ptr++ = d1;
    *Flash_ptr++ = d2;
    *Flash_ptr++ = d3;
    *Flash_ptr++ = d4;
  }
}

/******************************************************************************
**
**	Procedura SetVolume
**
******************************************************************************/ 
void SetVolume(unsigned char level)
{
  PrepareVolumeData(level);                     // nachysta posloupnost dat pro nastaveni el. potenciometru MCP4241
  WritePotNoIRQ();                              // zapise pomoci SPI data do potenciometru - bez preruseni (ceka na odeslani kazdeho byte)
}

/******************************************************************************
**
**	Procedura PrepareVolumeData
**
******************************************************************************/ 
void PrepareVolumeData(unsigned char volume)
{
  unsigned char vol;
  vol = TabLogVolume[volume];                   // prekoduje rozsah 0-40 pro logaritmicky prubeh pomoci tabulky hodnot (max rozsah MCP4241 je 128)
  PotTxData[0] = 0x00;                          // kod zapisu hodnoty pro wiper 0 bez ukladani do EEPROM obvodu MCP4241
  PotTxData[1] = vol;                           // hodnota volume pro pravy kanal
  PotTxData[2] = 0x10;                          // kod zapisu hodnoty pro wiper 1 bez ukladani do EEPROM obvodu MCP4241
  PotTxData[3] = vol;                           // hodnota volume pro levy kanal (stejne nastaveni pro oba kanaly)
}

/******************************************************************************
**
**	Procedura WritePotNoIRQ
**
******************************************************************************/ 
void WritePotNoIRQ(void)
{
  unsigned char i,j;
  IE2 &= ~UCA0RXIE;                             // zakaze USCI0 RX interrupt - pro pripad vyuzivani posilani dat i v preruseni 
  j = 0;                                        // inicializace ukazatele na pozadovany byte dat pro MCP4241
  for(i=0;i<2;i++)
  {
    while (!(IFG2 & UCA0TXIFG));                // USCI_A0 TX buffer prazdny?
    __delay_cycles(10);                         // ceka mezi vysilanim z duvodu indikace odvysilani dat drive nez ve skutecnosti
    P1OUT |= PotCS;                             // chip select obvodu neaktivni pro indikaci pripravy prijmu dalsich dat
    __delay_cycles(20);                         // chvili ceka aby zareagoval spravne MCP4241
    P1OUT &= ~PotCS;                            // aktivace CS k dalsimu prenosu dat do MCP4241
    UCA0TXBUF = PotTxData[j++];                 // vyslani dat a soucasne zvyseni ukazatele o +1
    while (!(IFG2 & UCA0TXIFG));                // USCI_A0 TX buffer prazdny?
    __delay_cycles(20);                         // chvili ceka mezi vysilanim
    UCA0TXBUF = PotTxData[j++];                 // odeslani dalsiho byte a inkrementace ukazatele o +1
  }
  while (!(IFG2 & UCA0TXIFG));                  // USCI_A0 TX buffer prazdny?
  __delay_cycles(30);                           // chvili ceka mezi vysilanim
  P1OUT |= PotCS;                               // deaktivace CS - 16bitu je pro jeden kanal potenciometru MCP4241
//  IE2 |= UCA0RXIE;                            // povoleni USCI0 RX interrupt zapomentovany - preruseni zatim nepouzivano
}

/******************************************************************************
**
**	Procedura Delay500ms
**
******************************************************************************/ 
void Delay500ms(void)
{
    TA1R = 0;                                   // nulovani citace 1
    WDTCTL = WDT_ARST_1000;                     // nulovani citace watchdogu
    while(!(TA1CCTL0&CCIFG));                   // wait 500ms
    TA1CCTL0 &= ~CCIFG;                         // ceka na docitani citace na pozadovanou hodnotu (CCR1)
}